/*
 *************************************************************************
 * Copyright (c) 2008, 2011 Actuate Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Actuate Corporation - initial API and implementation
 *  
 *************************************************************************
 */

package org.eclipse.birt.report.data.oda.jdbc.ui.editors;

import java.util.Properties;

import org.eclipse.birt.report.data.oda.jdbc.JDBCDriverManager;
import org.eclipse.datatools.connectivity.oda.IConnection;
import org.eclipse.datatools.connectivity.oda.IDriver;
import org.eclipse.datatools.connectivity.oda.OdaException;
import org.eclipse.datatools.connectivity.oda.design.DataSourceDesign;
import org.eclipse.datatools.connectivity.oda.design.ui.designsession.DesignSessionUtil;


public class OdaConnectionProvider
{
	private DataSourceDesign dataSourceDesign;
	private IConnection connection;
	
	OdaConnectionProvider( DataSourceDesign dataSourceDesign )
	{
		this.dataSourceDesign = dataSourceDesign;
	}
	
	IConnection openConnection( ) throws OdaException
	{
		if ( connection != null )
		{
			return connection;
		}
		IDriver jdbcDriver = JDBCDriverManager.getInstance().getDriver( dataSourceDesign.getEffectiveOdaExtensionId( ) );
		try 
		{
			connection = jdbcDriver.getConnection( dataSourceDesign.getEffectiveOdaExtensionId( ) );
			Properties prop = DesignSessionUtil.getEffectiveDataSourceProperties( dataSourceDesign );
			connection.open( prop );
		}
		catch ( OdaException e)
		{
			connection = null;
			throw e;
		}
		return connection;
	}
	
	void release( )
	{
		if ( connection != null )
		{
			try
			{
				connection.close( );
			}
			catch ( OdaException e )
			{
				
			}
			finally 
			{
				connection = null;
			}
		}
	}
	
}
